#define DEBUG 0
#include <Debug.h>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <Application.h>
#include <Roster.h>
#include <Alert.h>
#include <String.h>
#include <Path.h>
#include <Entry.h>
#include <FindDirectory.h>
#include <Directory.h>
#include <Node.h>
#include <fs_attr.h>

#include "MozillaOpenerApp.h"
#include "MozillaOpenerWindow.h"
#include "URL_file.h"

MozillaOpenerApp::MozillaOpenerApp	(const char * a_app_sig)
 : BApplication	(a_app_sig)	
{
	argv_received	=	false;
	refs_received	=	false;
		
//	if (CreateTempFolder() != B_OK)
//		exit(-1);	

	GetPreferences();

	m_window = 	new MOWindow (BRect(300,200,495,320));
}

MozillaOpenerApp::~MozillaOpenerApp	(void)	
{
	// void
}

void 
MozillaOpenerApp::ArgvReceived	(int32 argc, char **argv)
{
	argv_received	=	true;

	if(argc <= 1)	// no arguments
		return;

	for(int32 i = 1; i < argc; i++)
	{
		m_command = argv[i];
		m_command.Prepend(" ");
		m_command.Prepend(m_path);   // Path to Mozilla
		m_command.Append(" &");
		
		PRINT(("\"%s\"", m_command.String()));
		
		system(m_command.String());
	}
}

void 
MozillaOpenerApp::RefsReceived	(BMessage * a_message)
{
	// the mozilla launch script can only handle one file or URL
	// so this function only handles one, single file (entry_ref)
	
	refs_received	=	true;

	entry_ref	ref;
	if (a_message->FindRef("refs", 0, &ref) == B_OK)	
	{
		URL_file	file	(&ref);	
		if (file.InitCheck() == B_OK)
		{
			m_command	=	file.URL();
			m_command.Prepend(" ");
			m_command.Prepend(m_path);
			m_command.Append(" &");
			
			printf("%s\n", m_command.String());
			system(m_command.String());	
		}
		else
		{
			BEntry	entry	(&ref);
			BPath	path	(&entry);
		
			m_command	=	path.Path();
			makeShellSafe	(& m_command);
			m_command.Prepend(" ");
			m_command.Prepend(m_path);
			m_command.Append(" &");
			
			printf("%s\n", m_command.String());
			system(m_command.String());		
		}
	}	
}

void
MozillaOpenerApp::makeShellSafe	(BString * theString)
{
	int32	position = 0;
	while(position < theString->Length())
	{
		if(theString->ByteAt(position) == '\"')
		{	theString->Insert("\\",position);		position++;	}
		else if(theString->ByteAt(position) == '$')
		{	theString->Insert("\\",position);		position++;	}
		else if(theString->ByteAt(position) == '`')
		{	theString->Insert("\\",position);		position++;	}
		position++;
	}

	theString->Prepend("\"");
	theString->Append("\"");
}

void
MozillaOpenerApp::ReadyToRun	(void)
{
	if (argv_received || refs_received)
	{
		PostMessage(B_QUIT_REQUESTED);
		return;
	}

	m_window->Show();
}

void 
MozillaOpenerApp::MessageReceived	(BMessage * a_message)
{
	switch(a_message->what)
	{
		case MO_HELP_REQUESTED:
				Help();
				break;
				
		default:
				BApplication::MessageReceived(a_message);
				break;
	}
}

void
MozillaOpenerApp::AboutRequested	(void)
{
	BAlert * alert = new BAlert("MozillaOpener", "MozillaOpener 1.0\n\n"
	"Freeware   " B_UTF8_COPYRIGHT " Jonas Sundström @ kirilla.com\n", 
	"Help", "Kirilla", "Ok", B_WIDTH_AS_USUAL, B_INFO_ALERT);

	int32 reply = alert->Go();
	
	if (reply == 1)
	{
		BMessage msg(B_ARGV_RECEIVED); 
		msg.AddString("argv", "app"); 
		msg.AddString("argv", "http://www.kirilla.com/");  
		msg.AddInt32("argc", 2); 
		be_roster->Launch ("application/x-vnd.Be.URL.http", &msg ); 			
	}
	
	if (reply == 0)
		Help();
}

void 
MozillaOpenerApp::Help	(void)
{
	(new BAlert("MozillaOpener", 
	
	"MozillaOpener is a helper application that opens URLs, "
	"NetPositive Bookmarks, and HTML files in Mozilla.\n\n"

	"For MozillaOpener to work at all, it needs to know the "
	"location of Mozilla's launch script. In the MozillaOpener "
	"window, there is a text entry field named 'Mozilla is here', "
	"for this purpose.\n\n"
	
	"You also need to run the install scripts, to make MozillaOpener "
	"the preferred handler of URLs, Bookmarks and HTML files.\n\n"
	
	"You should now be able to double-click Bookmarks and HTML files, "
	"and have them open in Mozilla.",
	
	"Right", 
	NULL, NULL, B_WIDTH_AS_USUAL, B_IDEA_ALERT))->Go();
}

void 
MozillaOpenerApp::ProcessRefs	(BMessage * a_message)
{

}

void 
MozillaOpenerApp::GetPreferences(void)	
{
	// read preferences from app node

	// app ref
	app_info	my_app_info;
	be_app->GetAppInfo(& my_app_info);
	m_app_ref	=	my_app_info.ref;

	// app node
	BNode	node	(& m_app_ref);
	
	char *	buffer	=	new char [4096];
	off_t	bytes	=	0;

	// dir
	bytes = node.ReadAttr("MozillaOpener:path", B_STRING_TYPE, 0, buffer, 4095);
	if (bytes >= 0)
	{
		buffer[bytes] = '\0';
		m_path	=	buffer;
	}
	else
	{
		node.WriteAttr("MozillaOpener:path", B_STRING_TYPE, 0, "/boot/apps/mozilla/mozilla", 27);
		GetPreferences();
		return;
	}
	
	m_in_terminal	=	true;
	m_keep_open		=	false;
	
	delete [] buffer;
}

/*

void 
MozillaOpenerApp::RunScript(void)
{
	// create kickstart_script
	BString kickstart_script	("#!/bin/sh\n");

	// working dir
	kickstart_script.Append("cd ");	
	kickstart_script.Append(m_path);	
	kickstart_script.Append("\n");	

	// command
	kickstart_script.Append(m_command);
	kickstart_script.Append("\n");
	
	// keep open
	if (m_keep_open && m_in_terminal)
		kickstart_script.Append("read -p \"\n(Press Enter to close)\"\n");
	
	//	create tempfile name
	int32	counter	=	1;
	BString tempfile_name;
	tempfile_name << counter;
	while(m_tmp_dir->Contains(tempfile_name.String()))
	{
		counter	++;
		tempfile_name.Truncate(0);
		tempfile_name << counter;
	}
	
	// write kickstart script to file
	BFile	tmp_file;
	if (m_tmp_dir->CreateFile(tempfile_name.String(), & tmp_file) != B_OK)
		exit(-1);
		
	tmp_file.Write(kickstart_script.String(), kickstart_script.Length());
	
	// open Terminal with kickstart script
	
	BPath	tempfile_path	(m_tmp_dir, tempfile_name.String());
	BString command_string	(tempfile_path.Path());

	if (m_in_terminal)
	{
		command_string.Prepend(" ");
		command_string.Prepend(m_app_ref.name);
		command_string.Prepend("/boot/beos/apps/Terminal -t ");
	}
	
	command_string.Append(" &");
	
	PRINT(("system():\n%s\n", command_string.String()));
	system(command_string.String());	
}

status_t
MozillaOpenerApp::CreateTempFolder	(void)
{
	status_t status;
	BPath p;
	if (find_directory(B_COMMON_TEMP_DIRECTORY, &p) != B_OK)	return B_ERROR;
	
	m_tmp_dir	=	new BDirectory();
	BDirectory temp_dir;
	
	if ((status = m_tmp_dir->	SetTo(p.Path())) != B_OK) 	return status;
	status	=	m_tmp_dir->	CreateDirectory("Kirilla_MozillaOpener", &temp_dir);
	if (status != B_OK && status != B_FILE_EXISTS)	return status;

	p.SetTo(p.Path(),"Kirilla_MozillaOpener");
	if ((status = m_tmp_dir->	SetTo(p.Path())) != B_OK)	return status;
		
	BEntry entry(p.Path());
	if (entry.Exists()	&&	entry.IsDirectory())	return B_OK;
	else											return B_ERROR;
}
*/

